
	/* read input from a MONGO process - every time a line of the
	   form "position is now x y" appears, strip out the x and y
	   values and look for the closest match inside the given
	   .pht file.  Print out the closest line. */

#include <stdio.h>
#include <math.h>
#include <ctype.h>

#define LINELEN      256
#define MAX_STAR    1000		/* max number of stars in each list */

#define BIG       1.0e6			/* a number >> max number of rows, cols */
#define RADIUS      5.0			/* distance (in pixels) within which two */
                      			/*   stars count as 'matched' */
#define RAD2  (RADIUS*RADIUS)	/* square of the RADIUS distance */

struct s_star {
	int index;
	double row, col;
	char line[LINELEN];
};

static struct s_star star[MAX_STAR];

char *usage = "usage: showpht2 file.pht";

main(argc, argv)
int argc;
char *argv[];
{
	int done, num_star;
	char file[200], line[200];
	double x, y;
	FILE *fp;

	if (argc != 2) {
		fprintf(stderr, "%s\n", usage);
		exit(1);
	}
	strcpy(file, argv[1]);
	slurp(file, &num_star);
	done = 0;
	do {
		if (gets(line) == NULL)
			break;
		if ((strncmp(line, "end", 3) == 0) || (strncmp(line, "quit", 4) == 0))
			break;

		if (strncmp(line, ">position is now", 16) == 0) {
			if (sscanf(line + 16, "%lf %lf", &x, &y) != 2) {
				fprintf(stderr, "bad position line ..%s..\n", line);
				exit(1);
			}
			print_closest(x, y, num_star);
		}
	} while (1);

	exit (0);
}

	
	/* go through the star data looking for the star which 
	   is closest to the given x and y positions */

int
print_closest(x, y, num_star)
double x, y;
int num_star;
{
	int i, minstar;
	double min, dx, dy, dist;
	char buf[30];

	min = 1e6;
	minstar = 0;

	for (i = 0; i < num_star; i++) {
		dx = (star[i].row - y);
		dy = (star[i].col - x);
		dist = (dx*dx) + (dy*dy);
		if (dist < min) {
			min = dist;
			minstar = i;
		}
	}
	strncpy(buf, star[minstar].line + 2, 19);
	buf[19] = '\0';
	printf("%19s %-s\n", buf, (star[minstar].line + 38));
	fflush(stdout);
}


	/* read index, row, col values from a STARS or PHOT output
	   file into the given row of the array.  
	   place the number of items read into the passed parameter 'num_read'.

	   return 0 if all is well, 1 if there's a
	   problem. */

int
slurp(filename, num_read)
char *filename;
int *num_read;
{
	int i;
	char line[LINELEN];
	FILE *fp;

	if ((fp = fopen(filename, "r")) == NULL) {
		fprintf(stderr, "can't open file %s\n", filename);
		return(1);
	}
	i = 0;
	skip_header(fp);
	while (fgets(line, LINELEN, fp) != NULL) {
		if (sscanf(line, "%d %lf %lf", &(star[i].index),
				&(star[i].row), &(star[i].col)) != 3) {
			fprintf(stderr, "error reading the following line from file %s\n",
					filename);
			fprintf(stderr, "%s\n", line);
			return(1);
		}
		strcpy(star[i].line, line);
		i++;
	}
	*num_read = i;

	return(0);
}

	/* skip over any pseudo-FITS file header there might be */

int 
skip_header(fp)
FILE *fp;
{
	int done;
	long pos;
	char line[LINELEN];

	done = 0;
	pos = 0;
	while (done == 0) {
		pos = ftell(fp);
		if (fgets(line, LINELEN, fp) == NULL) {
			done = 1;
		}
		else if ((line[0] == '#') || (isalpha(line[0]))) {
			if (strncmp(line, "END", 3) == 0) {
				pos = ftell(fp);
				done = 1;
			}
		}
		else
			done = 1;
	}
	fseek(fp, pos, 0);
}
