
#include <stdio.h>
#include <math.h>
#include "bait.h"
#include "pf.h"
#include "solvepht.h"

#define IMFILE     "solvepht.img"
#define SIGFILE    "solvepht.sig"
#define OUTFILE    "solvepht.out"

static void markoff( /* int nbadim, *badim, nbadstar, *badstar, 
			nvarstar, *varstar */ );

char usage[] = "usage: solvepht [debug] [infile=] [mconst=] [outfile=] [imfile=] [sigfile=] \n                [badim=] [badstar=] [varstar=] [min=] [max=] \n";

main(argc, argv)
int argc;
char *argv[];
{
	int i, nn, nbadim, nbadstar, nvarstar, *badim, *badstar, *varstar;
	char infile[100], outfile[100], imfile[100], sigfile[100];
	double mconst, min, max;


	debug_flag = 0;
	nbadim = 0;
	nbadstar = 0;
	nvarstar = 0;
	min = -1.0e6;
	max = 1.0e6;
	if ((badim = (int *) calloc(argc, sizeof(int))) == NULL) {
		fprintf(stderr, "main: can't malloc for %d ints \n", argc);
		exit(-1);
	}
	if ((badstar = (int *) calloc(argc, sizeof(int))) == NULL) {
		fprintf(stderr, "main: can't malloc for %d ints \n", argc);
		exit(-1);
	}
	if ((varstar = (int *) calloc(argc, sizeof(int))) == NULL) {
		fprintf(stderr, "main: can't malloc for %d ints \n", argc);
		exit(-1);
	}
	mconst = MAG_CONST;
	strcpy(infile, "");
	strcpy(outfile, OUTFILE);
	strcpy(imfile, IMFILE);
	strcpy(sigfile, SIGFILE);

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "help") == 0) {
			fprintf(stderr, "%s\n", usage);
			exit(0);
		}
		if (strcmp(argv[i], "debug") == 0) {
			debug_flag = 1;
			continue;
		}
		if (strncmp(argv[i], "infile=", 7) == 0) {
			if (sscanf(argv[i] + 7, "%s", infile) != 1) {
				fprintf(stderr, "bad value for infile ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		if (strncmp(argv[i], "outfile=", 8) == 0) {
			if (sscanf(argv[i] + 8, "%s", outfile) != 1) {
				fprintf(stderr, "bad value for outfile ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		if (strncmp(argv[i], "imfile=", 7) == 0) {
			if (sscanf(argv[i] + 7, "%s", imfile) != 1) {
				fprintf(stderr, "bad value for imfile ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		if (strncmp(argv[i], "sigfile=", 8) == 0) {
			if (sscanf(argv[i] + 8, "%s", sigfile) != 1) {
				fprintf(stderr, "bad value for sigfile ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		if (strncmp(argv[i], "mconst=", 7) == 0) {
			if (sscanf(argv[i] + 7, "%lf", &mconst) != 1) {
				fprintf(stderr, "bad value for mconst ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		if (strncmp(argv[i], "badim=", 6) == 0) {
			if (sscanf(argv[i] + 6, "%d", &nn) != 1) {
				fprintf(stderr, "bad value for badim ..%s..\n", argv[i]);
				exit(-1);
			}
			badim[nbadim++] = nn;
			continue;
		}
		if (strncmp(argv[i], "badstar=", 8) == 0) {
			if (sscanf(argv[i] + 8, "%d", &nn) != 1) {
				fprintf(stderr, "bad value for badstar ..%s..\n", argv[i]);
				exit(-1);
			}
			badstar[nbadstar++] = nn;
			continue;
		}
		if (strncmp(argv[i], "varstar=", 8) == 0) {
			if (sscanf(argv[i] + 8, "%d", &nn) != 1) {
				fprintf(stderr, "bad value for varstar ..%s..\n", argv[i]);
				exit(-1);
			}
			varstar[nvarstar++] = nn;
			continue;
		}
		if (strncmp(argv[i], "min=", 4) == 0) {
			if (sscanf(argv[i] + 4, "%lf", &min) != 1) {
				fprintf(stderr, "bad value for min ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		if (strncmp(argv[i], "max=", 4) == 0) {
			if (sscanf(argv[i] + 4, "%lf", &max) != 1) {
				fprintf(stderr, "bad value for max ..%s..\n", argv[i]);
				exit(-1);
			}
			continue;
		}
		fprintf(stderr, "%s\n", usage);
		exit(1);
	}

	read_input(infile, min, max);
	markoff(nbadim, badim, nbadstar, badstar, nvarstar, varstar);

	if (debug_flag) {
		print_images(stdout);
		print_stars(stdout, LONG);
	}

	solve_matrix(mconst);

	write_images(imfile);
	write_sigma(sigfile);
	write_stars(outfile);

	exit(0);
}

	/* mark all the given stars and/or images as BAD, whih means they're
	   not used in the solution at all, or (stars only) as VARIABLE,
	   meaning that they're not used to solve for the em's, but later
	   do have their magnitudes calculated. */

static void 
markoff(nbadim, badim, nbadstar, badstar, nvarstar, varstar)
int nbadim, *badim, nbadstar, *badstar, nvarstar, *varstar;
{
	int i;
	struct s_starp *sp;

	for (i = 0; i < nbadim; i++) {
		if ((badim[i] < 0) || (badim[i] > num_images)) {
			fprintf(stderr, "markoff: nbadim = %d, badim[%d] = %d? \n", 
					nbadim, i, badim[i]);
			exit(-1);
		}
		image_arr[badim[i]].include = 0;
		for (sp = image_arr[badim[i]].starp; sp != NULL; sp = sp->next)
			sp->star->include = 0;
	}

	for (i = 0; i < nbadstar; i++) {
		if ((badstar[i] < 0) || (badstar[i] > num_ids)) {
			fprintf(stderr, "markoff: nbadstar = %d, badstar[%d] = %d? \n", 
					nbadstar, i, badstar[i]);
			exit(-1);
		}
		star_arr[badstar[i]].include = 0;
	}

	for (i = 0; i < nvarstar; i++) {
		if ((varstar[i] < 0) || (varstar[i] > num_ids)) {
			fprintf(stderr, "markoff: nvarstar = %d, varstar[%d] = %d? \n", 
					nvarstar, i, varstar[i]);
			exit(-1);
		}
		star_arr[varstar[i]].variable = 1;
	}
}
