
	/* definitions for the "solvepht" program */	


#define MAX_STAR  5000		/* max number stars total we can have */
#define STRLEN    40		/* max length of star names and filters */
#define LINELEN   256
#define DEF_APER  -1		/* default aperture used for measurements */
                   			/*    note we use (aper-1) to access arrays */
                   			/*    so DEF_APER should start at 1, not 0 */
                   			/*    value of -1 means "use fwhm to pick aper" */
#define MAX_APER  10		/* max number of apertures a star may have */
#define LINELEN   256		/* max length of data lines */
#define NODATA    99.0		/* value assigned to a variable w/ no data */
#define PHT_POS   38		/* position in a .pht data line at which */
                    		/*    mag/err pairs begin */
#define PHT_LEN   15		/* length of a mag/err pair in chars */
#define MAG_CONST 0.0		/* def value for reported mag of brighest star */

#define MJD(jd)  ((jd) - 2440000.0)

	/* parameters used to determine if stars match or not */
#define BIG       1.0e6			/* a number >> max number of rows, cols */
#define RADIUS      2.0			/* distance (in pixels) within which two */
                      			/*   stars count as 'matched' */
#define RAD2  (RADIUS*RADIUS)	/* square of the RADIUS distance */

	/* some flags which tell the program its function */
#define PRINT     1			
#define AUTO      2			/* don't ask for input from user */


	/* various flags for printing */
#define SHORT     1
#define LONG      2

struct s_image {
	int index;				/* identifying index used in output routines */
	char name[STRLEN];
	char filter[STRLEN];
	char object[STRLEN];
	double exptime;
	double airmass;
	double ut;
	double mjd;
	int num_star;			/* number of stars in the image */
	struct s_starp *starp;	/* linked list of stars in this image */
	int num_match;			/* # stars that match template image */
	int include;			/* if 1 include stars from this image; if 0 don't */
};

	/* this is the structure which records all the information about
	   one instance of a star on a single image */
struct s_star {
	int id;						/* ID number in set of all images' stars */
	struct s_image *image;		/* this star's image */
	double xc;
	double yc;
	double peak;				/* peak value above sky, from .coo file */
	double fwhm;				/* fwhm of star, from .coo file */
	double sharp;				/* sharpness of star, from .coo file */
	double round;				/* roundness of star, from .coo file */
	double mag;
	double weight;
	int include;				/* yes/no weight for this star, this image */
	int saturated;				/* yes/no star is (possibly) saturated */
	struct s_star *next;
};

	/* and this is the structure which records all the information about
	   ALL the instances of the SAME star, from many images */
struct s_allstar {
	double xc, yc;
	int id;
	int num;					/* number of stars with this ID */
	struct s_star *stars;		/* a linked list of stars with this ID */
	int include;				/* yes/no include star w/this ID in solution */
	int variable;				/* yes/no treat this star as a variable */
};

struct s_starp {
	struct s_star *star;
	struct s_starp *next;
};

	/* these are the heads of some important arrays/lists
	             image_arr: an array of all the images
	             star_list: a list of all the instances of the same star
	 */

struct s_image *image_arr;
struct s_allstar *star_arr;

	/* some global variables */

int num_images;			/* total number of images */
int num_ids;			/* total number of unique stars */
int debug_flag;

	/* some global functions */
