

#include <stdio.h>
#include <math.h>
#include "bait.h"
#include "pf.h"
#include "solvepht.h"

static void eliminate( /* */ );
static void read_it( /* FILE *fp */ );
static int good_images( /* */ );
static int good_stars( /* */ );
static void read_stars( /* FILE *fp */ );
static int read_one_star( /* FILE *fp; struct s_allstar *p */ );
static void read_images( /* FILE *fp */ );
static int read_one_image( /* FILE *fp; struct s_image *im */ );
static void check_names( /* */ );
static void check_mags( /* double min, max */ );

	/* read information on all 'acceptable' images and stars from
	   the given file (or stdin, if passed an empty string "").
	   any star with ANY instrumental mag < min or > max is marked as
	   'unacceptable'. */

read_input(file, min, max)
char *file;
double min, max;
{
	int i, naper, pos;
	char line[PF_LEN + 1];
	FILE *fp;

	if (strcmp(file, "") == 0)
		fp = stdin;
	else if ((fp = fopen(file, "r")) == NULL) {
		fprintf(stderr, "read_input: can't open file %s for reading\n", file);
		exit(-1);
	}

	read_it(fp);
	fclose(fp);

	check_names();
	check_mags(min, max);	

}

	/* set to zero the weights of all stars which are either
	   in an 'unacceptable' image, or an instance of an 'unacceptable'
	   star.  */

static void 
eliminate()
{
	int i, j;
	struct s_star *s;
	struct s_starp *spp;

	for (i = 0; i < num_images; i++)
		if (image_arr[i].include == 0)
			for (spp = image_arr[i].starp; spp != NULL; spp = spp->next)
				spp->star->include = 0;

	for (i = 0; i < num_ids; i++)
		if (star_arr[i].include == 0)
			for (s = star_arr[i].stars; s != NULL; s = s->next)
				s->include = 0;

}



	/* the actual reading routine */

static void 
read_it(fp)
FILE *fp;
{

	/* there are two kinds of information to print out: information 
	   about images (read into the 'image_arr[]' array), and information
	   about stars (read into the 'star_arr[]' array). 
	   the first thing to do is to read how many images and how many
	   stars will appear in the input. */

	if (fscanf(fp, "%d images  %d stars \n", &num_images, &num_ids) != 2) {
		fprintf(stderr, "read_it: invalid first line of input file \n");
		exit(-1);
	}

	/* now read information on each image: get one line per image */
	read_images(fp);	

	/* and now read information on each star: get N+1 lines per star,
	   where N is the number of images of each star */
	read_stars(fp);

}

	/* calculate the number of 'good' images, and return it */

static int 
good_images()
{
	int i, sum;

	sum = 0;
	for (i = 0; i < num_images; i++)
		if (image_arr[i].include != 0) 
			sum++;
	return(sum);
}

	/* calculate the number of 'good' stars, and return it.  Make sure
	   that there is at least one 'good' instance of each star. */

static int
good_stars()
{
	int i, sum;
	struct s_star *star;

	sum = 0;
	for (i = 0; i < num_ids; i++) {
		if (star_arr[i].include != 0) {
			for (star = star_arr[i].stars; star != NULL; star = star->next) {
				if ((star->weight != 0) || (star->include != 0)) {
					sum++;
					break;
				}
			}
		}
	}
	
	return(sum);
}

	/* read information on each star: they should be in order 
	   by master list x-coordinate */

static void
read_stars(fp)
FILE *fp;
{
	int i;

	if ((star_arr = (struct s_allstar *) 
				calloc(num_ids, sizeof(struct s_allstar))) == NULL) {
		fprintf(stderr, "read_stars: can't calloc for star_arr array \n");
		exit(-1);
	}
	for (i = 0; i < num_ids; i++) {
		read_one_star(fp, &(star_arr[i]));
	}	
}

	/* read the pertinent information from a single star entry
	   (which records all instances of the same star) from the given 
	   FILE pointer.  Each 'master star' gets N+1 lines of output,
	   where N is the number of actual instances of the star.  The first
	   line includes the number N at its end.  */

static int
read_one_star(fp, allstar)
FILE *fp;
struct s_allstar *allstar;
{
	int i, index;
	struct s_star *p, *star;
	struct s_starp *sp, *spp;

	if (fscanf(fp, "%d %lf %lf %d \n", &(allstar->id), &(allstar->xc), 
						&(allstar->yc), &(allstar->num)) != 4) {
		fprintf(stderr, "read_one_star: can't read first line\n");
		exit(-1);
	}
	allstar->include = 1;
	allstar->variable = 0;
	allstar->stars = (struct s_star *) NULL;

	for (i = 0; i < allstar->num; i++) {
		if ((star = (struct s_star *) malloc(sizeof(struct s_star))) == NULL) {
			fprintf(stderr, 
					"read_one_star: can't malloc for instance %d of star %d\n",
							 i, allstar->num);
			exit(-1);
		}
		if (fscanf(fp, "  %d %d %d %lf %lf %lf %lf %lf %lf\n",
						&(star->id), &(index), &(star->saturated) , 
						&(star->peak), &(star->fwhm), &(star->sharp),
						&(star->round), &(star->mag), &(star->weight)) != 9) {
			fprintf(stderr, 
				"read_one_star: can't read instance %d of star %d\n",
						i, allstar->num);
			exit(-1);
		}
		star->include = 1;
		if ((index < 0) || (index >= num_images)) {
			fprintf(stderr, "star with ID %d has invalid image index %d\n",
					star->id, index);
			exit(-1);
		}
		star->image = &(image_arr[index]);
		star->next = (struct s_star *) NULL;

		/* walk down the list of linked stars belonging to 'allstar' and
		   insert the new one at the end */
		if (allstar->stars == NULL)
			allstar->stars = star;
		else {
			for (p = allstar->stars; p->next != NULL; p = p->next)
				;
			p->next = star;
		}
			
		/* and now insert this star into the linked list dangling from
		   its image */
		if ((sp = (struct s_starp *) malloc(sizeof(struct s_starp))) == NULL) {
			fprintf(stderr, 
				"read_one_star: can't malloc for starp, instance %d of star %d\n",
							 i, allstar->num);
			exit(-1);
		}
		sp->star = star;
		sp->next = (struct s_starp *) NULL;
		if (star->image->starp == NULL) 
			star->image->starp = sp;
		else {
			for (spp = star->image->starp; spp->next != NULL; spp = spp->next)
				;
			spp->next = sp;
		}

	}
}


	/* read in information on each image, which should be in order 
	   by date and time of observation */

static void
read_images(fp)
FILE *fp;
{
	int i;

	if ((image_arr = (struct s_image *)
				calloc(num_images, sizeof(struct s_image))) == NULL) {
		fprintf(stderr, "read_images: can't calloc for %d images\n",
				num_images);
		exit(-1);
	}

	for (i = 0; i < num_images; i++) {
		read_one_image(fp, &(image_arr[i]));
	}	
}

	/* read the pertinent information in a single image structure from
	   the give FILE pointer */

static int
read_one_image(fp, im)
FILE *fp;
struct s_image *im;
{

	if (fscanf(fp, "%d %s %s %s %lf %lf %lf %lf %d %d\n",
			&(im->index), im->name, im->object, im->filter, &(im->exptime), 
			&(im->airmass), &(im->ut),
			&(im->mjd), &(im->num_star), &(im->num_match)) != 10) {
		fprintf(stderr, "read_one_image: can't read data for image\n");
		exit(-1);
	}
	im->include = 1;
	im->starp = (struct s_starp *) NULL;

}

	/* print warning messages if an image's object name and/or filter does 
	   not match the first image's */

static void
check_names()
{
	int i, j;

	for (i = 1; i < num_images; i++) {
		if (strcmp(image_arr[i].object, image_arr[0].object) != 0) {
			fprintf(stderr, "check: image %3d ..%s.. doesn't match image %3d ..%s..\n", 
					image_arr[i].index, image_arr[i].object, image_arr[0].index,
					image_arr[0].object);
		}
		if (strcmp(image_arr[i].filter, image_arr[0].filter) != 0) {
			fprintf(stderr, "check: image %3d filter ..%s.. doesn't match image %3d ..%s..\n", 
					image_arr[i].index, image_arr[i].filter, image_arr[0].index,
					image_arr[0].filter);
		}
	}
}

	/* set to zero the 'include' field of any star with at an average
	   magnitude (average of all actual data values) which is outside
	   the given input instrumental magnitude limits */

static void 
check_mags(min, max)
double min, max;
{
	int i, num;
	double sum;
	struct s_star *s;

	for (i = 0; i < num_ids; i++) {
		sum = 0.0;
		num = 0;
		if (star_arr[i].include == 1) {
			for (s = star_arr[i].stars; s != NULL; s = s->next) {
				if (s->mag != NODATA) {
					sum += s->mag;
					num++;
				}
			}
			sum /= (double) num;			
			if ((sum < min) || (sum > max)) {
				star_arr[i].include = 0;
			}
		}
	}
}

