
	/* given a file which consists of a pseudo-FITS header and then
	   some stuff, remove the header from the file so that only the stuff
	   is left. 

	            subheader file  [ outfile=newfile ]

	   if the 'outfile' flag is present, then a new file with the given name
	   will be created for the header-less file.  Otherwise, the original
	   file is overwritten.
*/

#include <stdio.h>
#include <ctype.h>
#include "pf.h"
#include "header.h"

char err_msg[BIG_BUF];
char usage[] = "usage: subheader file [ outfile= ] ";

main(argc, argv)
int argc;
char *argv[];
{
	int type;
	char outfile[BIG_BUF], *gotit, *find();
	
	strcpy(outfile, "");

	if ((argc != 2) && (argc != 3)) {
		fprintf(stderr, "%s\n", usage);
		exit(1);
	}
	if (argc == 3) {
		if ((gotit = find("outfile", argv[2])) == NULL) {
			fprintf(stderr, "%s\n", usage);
			exit(1);
		}
		strcpy(outfile, gotit);
	}
		
	if ((type = fits_file_type(argv[1])) != PSEUDO_FITS) {
		fprintf(stderr, "file %s does not have a pseudo-FITS header\n", 
						argv[1]);
		exit(1);
	}
	strip_header(argv[1], outfile);
}

	/* copy all the header information from a pseudo-FITS file into
	   a pseudo-FITS form (with newlines at the end of each 80-char
	   line) at the beginning of the given file */

int
strip_header(fitsfile, outfile)
char *fitsfile, *outfile;
{
	FILE *fits_fp, *out_fp;
	char line[BIG_BUF + 1];
	
	if ((fits_fp = fopen(fitsfile, "r")) == NULL) {
		sprintf(err_msg, "can't open pseudo-FITS file %s\n", fitsfile);
		die(err_msg);
	}
	if (strcmp(outfile, "") != 0) {
		if ((out_fp = fopen(outfile, "w")) == NULL) {
			sprintf(err_msg, "can't open output file %s\n", outfile);
			die(err_msg);
		}
	}
	else if ((out_fp = fopen(TEMPFILE, "w")) == NULL) {
		sprintf(err_msg, "can't open temporary file %s\n", TEMPFILE);
		die(err_msg);
	}

	line[BIG_BUF] = '\0';
	while (fgets(line, BIG_BUF, fits_fp) != NULL) {
		if (strncmp(line, "END", 3) == 0)
			break;
	}
	while (fgets(line, BIG_BUF, fits_fp) != NULL) {
		fprintf(out_fp, "%s", line);
	}
	fclose(fits_fp);
	fclose(out_fp);

	/* if an output file wasn't specified, now it's time for the ugly 
	   copying operation */
	if (strcmp(outfile, "") != 0) 
		return(0);

	if ((fits_fp = fopen(fitsfile, "w")) == NULL) {
		sprintf(err_msg, "can't open pseudo-FITS file %s for overwriting\n", 
					fitsfile);
		die(err_msg);
	}
	if ((out_fp = fopen(TEMPFILE, "r")) == NULL) {
		sprintf(err_msg, "can't open temporary file %s\n", TEMPFILE);
		die(err_msg);
	}
	rewind(out_fp);
	while (fgets(line, BIG_BUF, out_fp) != NULL)		
		fprintf(fits_fp, "%s", line);
	fclose(fits_fp);
	fclose(out_fp);
	if (unlink(TEMPFILE) < 0) {
		fprintf(stderr, "can't unlink temp file %s\n", TEMPFILE);
	}

	return(0);
}

