
	/* translate a file from a format like that of "dofile" to one
	   that 'listgen' can read. The required fields are RA and
	   Dec (assumed epoch 1950.0) in the first columns of the file,
	   like so:
	            00 07 18.6 +23 14 05
	            01 50 12   -23 05      
	   where the first column starts in the first character of each line,
	   there is a single space between each field, and the last field
	   (the Dec seconds) may be blank - denoting zero. 

	   exposure times, if given, are based on a 30 second exposure for 
	   a mag 10.0 object. */

#include <stdio.h>
#include <math.h>

#define LINELEN 100

main(argc, argv)
int argc;
char *argv[];
{
	char line[LINELEN];
	double x;
	int rah, ram, decd, decm, decs;
	double ras, mag, exp_time;
	FILE *fp, *fopen();
	
	if (argc > 1) {
		if ((fp = fopen(argv[1], "r")) == NULL) {
			fprintf(stderr, "can't open file %s for reading \n", argv[1]);
			exit(1);
		}
	}
	else
		fp = stdin;

	while (fgets(line, LINELEN, fp) != NULL) {
		if ((line[0] == '\n') || (line[0] == ' ') || (line[0] == '\t'))
			continue;
		if (sscanf(line,      "%2d", &rah) != 1) {
			fprintf(stderr, "bad line format: ..%s..\n", line);
			exit(1);
		}
		if (sscanf(line + 3,  "%2d", &ram) != 1) {
			fprintf(stderr, "bad line format: ..%s..\n", line);
			exit(1);
		}
		if (sscanf(line + 6,  "%4lf", &ras) != 1) {
			fprintf(stderr, "bad line format: ..%s..\n", line);
			exit(1);
		}
		if (sscanf(line + 12, "%3d", &decd) != 1) {
			fprintf(stderr, "bad line format: ..%s..\n", line);
			exit(1);
		}
		if (sscanf(line + 16, "%2d", &decm) != 1) {
			fprintf(stderr, "bad line format: ..%s..\n", line);
			exit(1);
		}
		if (sscanf(line + 19, "%2d", &decs) != 1)
			decs = 0.0;

		if (sscanf(line + 40, "%4lf", &mag) != 1)
			mag = 15.0;

		/* convert (could handle precession here...) */
		exp_time = 30*pow(10.0, 0.2*(mag - 10.0));

		/* and here comes the output */
		printf("%02d%02d%06.3lf %+02d%02d%02d.000 1950.0 %6.1lf 5 0000 0030 ",
				rah, ram, ras, decd, decm, decs, exp_time);
		printf("-10000.0 %2d -1 0 1.0 \n", (rand()/32767)*10);
	}
}
