
	/* a simple little program that just prints out the UT date of the next
	   or current observing night, in the form
	                       DD/MM/YYYY
	   other programs can compare this date to the DAYSTART and DAYEND
	   values in a request header. */

/*	modified days to put a placeholder in month 0 
	11/30/91 - RRT
	2/19/92 - added 'string' feature to print out date string
		like Feb19 for use in filenames
	7/21/92 - allowed +xx to specify an offset in days from present  -rrt
*/
#include <stdio.h>
#include <math.h>
#include "bait.h"

char *progname = "utday";

static int days[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
static char	*mos[]= {"","Jan", "Feb","Mar","Apr","May","Jun","Jul",
						"Aug", "Sep", "Oct", "Nov", "Dec"};

static void usage();
static void another_get_today();

main(argc,argv)
int argc;
char *argv[];
{
	int day, month, year,offset=0;
	double ut, jd, jds, jde;
	int i, string_flag=0;

	for(i= 1; i< argc; i++){
		if(strcmp(argv[i],"help") == 0){
			usage();
			exit(0);
		}
		if(strcmp(argv[i],"string") ==0 ){
			string_flag=1;
			continue;
		}
		if(argv[i][0] == '+'){
			offset=atoi(argv[i]);
			continue;
		}
		fprintf(stderr, "unknown keyword '%s'\n",argv[i]);
		usage();
		exit(1);
	}

	/* first, figure out the current JD */
	another_get_today(&day, &month, &year,offset);
	get_ut(&ut);
	get_jd(day, month, year, ut, &jd);

	/* now determine the JD limits of the current or next night */
	night_limits(jd, -TWI_DEGREES, &jds, &jde);

	/* use the UT date corresponding to the JD of the START of night 
	   as the returned date */
	if ((floor(jd + 0.5) - 0.5) < (floor(jds + 0.5) - 0.5)) {
		if ((year % 4 == 0) && (year % 400 != 0))
			days[1] = 29;			/* February has 29 days in leap year */
		if (++day > days[month]) {
			day = 1;
			if (++month > 12) {
				month = 1;
				year++;
			}
		}
	}

	if(string_flag)
		printf("%s%2.2d\n",mos[month],day);
	else
		printf("%02d/%02d/%04d\n", day, month, year);
	exit (0);
}

	/* put the values of the current Greenwich day, month and year into 
	   the passed parameters 

	   With the number of days to offset it kludged by rrt
	   */

#include <time.h>
#define NSEC_DAY 86400

static void
another_get_today(day, month, year,offset)
int *day, *month, *year,offset;
{
	long t, time();
	struct tm *tm;

	t = time((long *) 0);
	t += (long)offset*NSEC_DAY;		/* since t is in seconds */
	tm = gmtime(&t);
	*day = tm->tm_mday;
	*month = tm->tm_mon + 1;		/* gmtime puts month in range 0-11 */
	*year = tm->tm_year + 1900;
}

	/* set the ut - used only when simulating observations */
			
static void usage()
{
	printf("usage: utday [help] [string] [+offset]\n");
}
